/****************************************************************
* AES DEMO PROGRAM
* Copyright 1992 by Clayton Walnum
* Developed with Laser C
*
* NOTES: This program is intended to illustrate the basic
*        techniques for handling menu bars, dialog boxes,
*        and windows. However, for the sake of clarity, it
*        takes certain liberties with the rules of GEM
*        programming. For example, one should not, as is done
*        here, use a "hard-coded" window handle when redrawing 
*        a window, since it is often impossible to know which 
*        window actually needs redrawing. Also, doing redraws
*        properly requires traversing GEM's rectangle list,
*        something which, for the sake of simplicity, is not
*        here.
****************************************************************/

#include <osbind.h>
#include <gemdefs.h>
#include <obdefs.h>

#include "aesdemo2.h"

#define TRUE         1
#define FALSE        0
#define YES          1
#define NO           2
#define OK           1
#define LEFT_BUTTON  0x0001
#define BUTTON_DOWN  0x0001
#define NUM_CLICKS   2
#define PARTS        NAME|INFO|UPARROW|DNARROW|VSLIDE|MOVER|FULLER|CLOSER|SIZER|HSLIDE
#define MEDIUM       1
#define HIGH         2
#define MIN_WIDTH    64
#define MIN_HEIGHT   64

int work_in[11], work_out[57], contrl[12], intin[128], ptsin[128], 
    intout[128], ptsout[128], msg_buf[8];

int handle, dum, key, fullx, fully, fullw, fullh, wrkx, wrky,
    wrkw, wrkh, oldx, oldy, oldw, oldh, curx, cury, curw, curh,
    w_h1, res, closed, charw, charh, all_done, mouse_x, mouse_y, 
    num_clicks;

char *string, filename[15], path[128];

OBJECT *menu, *dialog;

char *get_tedinfo_str ();

TEDINFO *ob_tedinfo;

/****************************************************************
* Main program
****************************************************************/
main ()
{
   appl_init ();            /* Initialize application.    */
   open_vwork ();           /* Set up workstation.        */
   do_demo();               /* Go do the demo.            */
   v_clsvwk (handle);       /* Close virtual workstation. */
   appl_exit ();            /* Back to the desktop.       */
}


/****************************************************************
* do_demo
*
* This function comprises the main program initialization, in
* which the resolution is checked, the resource file is loaded,
* the menu is displayed, and the windows are created and
* displayed. When the get_event() function returns, this
* function also closes and releases all program resources.
****************************************************************/
do_demo()
{
   if ( (res = Getrez ()) != HIGH && res != MEDIUM )
      form_alert(1,"[0][This demo runs|only in high or medium |resolution.][OK]");
   else {
      graf_mouse ( ARROW, 0L );

      if ( !rsrc_load ( "\AESDEMO2.RSC" ) )
         form_alert ( 1, "[1][AESDEMO2.RSC missing!][Okay]" );
      else {
         rsrc_gaddr ( R_TREE, MENU, &menu );
         rsrc_gaddr ( R_TREE, DIALOG, &dialog );
         menu_bar ( menu, TRUE );
         wind_get ( 0, WF_WORKXYWH, &fullx, &fully, &fullw, &fullh );
         w_h1 = wind_create ( PARTS, fullx, fully, fullw, fullh );
         wind_set ( w_h1, WF_NAME, "Window Name", 0, 0 );
         wind_set ( w_h1, WF_INFO, "Window Info Line", 0, 0 );
         wind_open ( w_h1, fullx, fully, fullw, fullh );
         wind_set ( w_h1, WF_VSLSIZE, 500, 0, 0, 0 );
         wind_set ( w_h1, WF_HSLSIZE, 500, 0, 0, 0 );
         wind_set ( w_h1, WF_HSLIDE, 1, 0, 0, 0 );
         wind_set ( w_h1, WF_HSLIDE, 1, 0, 0, 0 );
         menu_ienable ( menu, OPEN, FALSE );
         closed = FALSE;

         get_event ();

         menu_bar ( menu, FALSE );
         if (!closed) wind_close ( w_h1 );
         wind_delete ( w_h1 );
         rsrc_free ();
      }
   }
}


/****************************************************************
* get_event
*
* This is the main program loop. It continues to gather events
* until the user elects to quit the program. In this sample
* program, only message-type events are processed.
****************************************************************/
get_event ()
{
   int event;

   all_done = FALSE;
   while ( !all_done ) {
      event = evnt_multi ( MU_MESAG, NUM_CLICKS, 
                         LEFT_BUTTON, BUTTON_DOWN,
                         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, msg_buf, 0, 0,
                         &mouse_x, &mouse_y, &dum, &dum, &key, &num_clicks );

      if ( event & MU_MESAG )
         handle_messages ();

   }
}


/****************************************************************
* handle_messages
*
* This function routes whatever message was received to the
* proper function for processing.
****************************************************************/
handle_messages ()
{
   switch ( msg_buf[0] ) {

      case MN_SELECTED:
         do_menu ();
         break;

      case WM_REDRAW:
         do_redraw ();
         break;

      case WM_FULLED:
         do_full ( w_h1 );
         break;

      case WM_MOVED:
      case WM_SIZED:
         do_move();
         break;

      case WM_CLOSED:
         do_close ();
         break;
   }     
}


/****************************************************************
* do_menu
*
* This function translates menu messages and transfers program
* execution to the appropriate place.
****************************************************************/
do_menu ()
{
   switch ( msg_buf[3] ) {

      case DESK:
         form_alert(1,"[0][       AES Demo|   by Clayton Walnum   ][CONTINUE]");
         break;

      case FILE:
         switch ( msg_buf[4] ) {

            case OPEN:
               do_open ();
               break;

            case CLOSE:
               do_close ();
               break;

            case MDIALOG:
               do_dialog ();
               break;

            case QUIT:
               all_done = TRUE;
               break;
        }
        break;
   }
   menu_tnormal ( menu, msg_buf[3], TRUE );
}


/****************************************************************
* do_full
*
* This function handles the window's fuller button. If the
* window is already full sized, it sets the window to its
* previous size. If the window is less than full sized, this
* function sets the window size to the full screen.
****************************************************************/
do_full ( w_h )
int w_h;
{
   int c_x, c_y, c_w, c_h,
       f_x, f_y, f_w, f_h;

   wind_get ( w_h, WF_CURRXYWH, &c_x, &c_y, &c_w, &c_h );
   wind_get ( w_h, WF_FULLXYWH, &f_x, &f_y, &f_w, &f_h );
   if ( c_x != f_x || c_y != f_y || c_w != f_w || c_h != f_h ) {
      wind_get ( w_h1, WF_CURRXYWH, 
                 &curx, &cury, &curw, &curh );
      graf_growbox ( curx, cury, curw, curh,
                     fullx, fully, fullw, fullh );
      wind_set ( msg_buf[3], WF_CURRXYWH, 
                 fullx, fully, fullw, fullh );
   }
   else {
      wind_get ( w_h1, WF_PREVXYWH, 
                &oldx, &oldy, &oldw, &oldh );
      graf_shrinkbox ( oldx, oldy, oldw, oldh,
                      fullx, fully, fullw, fullh );
      wind_set ( msg_buf[3], WF_CURRXYWH, 
                oldx, oldy, oldw, oldh );
   }
}


/****************************************************************
* do_move
*
* This function handles the window's mover bar and sizer button,
* displaying the window in its new size or location. This function
* also enforces a minimum size on the window.
****************************************************************/
do_move()
{
   if ( msg_buf[6] < MIN_WIDTH )
      msg_buf[6] = MIN_WIDTH;
   if ( msg_buf[7] < MIN_HEIGHT )
      msg_buf[7] = MIN_HEIGHT;
   wind_set ( msg_buf[3], WF_CURRXYWH,
              msg_buf[4], msg_buf[5], msg_buf[6], msg_buf[7] );
}


/****************************************************************
* do_close
*
* This function closes the window and sets the FILE menu entries
* so that the OPEN entry is activated and the CLOSE entry is
* disabled.
****************************************************************/
do_close ()
{
   int button;

   button = form_alert ( 1, "[2][Do you want to close|this window?][YES|NO]");
   if ( button == YES ) {
      wind_close ( w_h1 );
      closed = TRUE;
      menu_ienable ( menu, OPEN, TRUE );
      menu_ienable ( menu, CLOSE, FALSE );
   }
}


/****************************************************************
* do_open
*
* This function opens the window, first allowing the user to
* select a file for the window's name from the file selector.
* This function also sets the FILE menu entries so that the OPEN
* entry is disabled and the CLOSE entry is activated.
****************************************************************/
do_open ()
{
   int button;
   char strg[128];

   button = select_file ( path, filename, "*.*", FALSE);
   if ( button == OK ) {
      strcpy ( strg, path );
      strcat ( strg, filename );
      wind_set ( w_h1, WF_NAME, strg, 0, 0 );
      wind_open ( w_h1, fullx, fully, fullw, fullh );
      menu_ienable ( menu, OPEN, FALSE );
      menu_ienable ( menu, CLOSE, TRUE );
      closed = FALSE;
   }
}


/****************************************************************
* do_dialog
*
* This function initializes and displays the dialog box, getting
* the user's response from the dialog buttons before closing the
* dialog and returning to the main program.
****************************************************************/
do_dialog ()
{
   int choice, okay;
   int dial_x, dial_y, dial_w, dial_h;

   string = get_tedinfo_str (dialog, NAMEEDIT );
   string[0] = '@';
   form_center ( dialog, &dial_x, &dial_y, &dial_w, &dial_h );
   form_dial ( FMD_START, 0, 0, 10, 10, dial_x, dial_y, dial_w, dial_h );
   form_dial ( FMD_GROW, 0, 0, 10, 10, dial_x, dial_y, dial_w, dial_h );
   objc_draw ( dialog, 0, 8, dial_x, dial_y, dial_w, dial_h );

   okay = FALSE;

   do {
      choice = form_do ( dialog, NAMEEDIT );
      dialog[choice].ob_state = SHADOWED;

      switch ( choice ) {

         case OKBUTN:
            form_alert(1,"[0][You clicked OK ][CONTINUE]");
            break;

         case CANCLBTN:
            form_alert(1,"[0][You clicked CANCEL ][CONTINUE]");
            break;
       }
   }
   while ( (choice != OKBUTN) && (choice != CANCLBTN) );

   form_dial ( FMD_SHRINK, 0, 0, 10, 10, dial_x, dial_y, dial_w, dial_h );
   form_dial ( FMD_FINISH, 0, 0, 10, 10, dial_x, dial_y, dial_w, dial_h );
}


/****************************************************************
* select_file
*
* This function initializes and displays the GEM file selector
* box. After the user exits from the file selector, this function
* constructs the path string.
****************************************************************/
select_file ( path, fnme, deflt, display)
char *path, *fnme, *deflt;
int display;
{
   int x, choice;
   char ch;

   if ( display == FALSE )
      for ( x=0; x<20; fnme[x++] = '\0' );

   Dgetpath ( path, 0 );
   strcpy ( &path[strlen(path)], "\\" );
   strcpy ( &path[strlen(path)], deflt );

   fsel_input ( path, fnme, &choice );

   x = strlen ( path );
   while ( ( ch = path[--x] ) != '\\' && x > 0 );
   path[x+1] = '\0';

   return ( choice );
}


/****************************************************************
* do_redraw
*
* This function draws the window's workspace. Note that the
* method used here will work only with a single-window program.
* Multiple-window programs need to get a window's handle from
* msgbuf, since that's the only way to know which window needs
* redrawing. Also, in a full program more than just a white
* rectangle needs to be drawn in a window; any data that must
* be displayed must also be drawn.
****************************************************************/
do_redraw ()
{
   GRECT r;

   wind_update ( BEG_UPDATE );
   graf_mouse ( M_OFF, 0L );
   vswr_mode ( handle, MD_REPLACE );
   wind_get ( w_h1, WF_WORKXYWH, &r.g_x, &r.g_y, &r.g_w, &r.g_h );
   draw_rec ( r, 2, 8, WHITE );
   wind_update ( END_UPDATE );
   graf_mouse ( M_ON, 0L );
}


/****************************************************************
* draw_rec
*
* This function draws a rectangle of the fill style and color
* requested. 
****************************************************************/
draw_rec ( rec, inter, fill, color )
GRECT rec;
int inter, fill, color;
{
   int pxy[4];

   graf_mouse ( M_OFF, 0L );
   vsf_interior ( handle, inter );
   vsf_style ( handle, fill );
   vsf_color ( handle, color );
   pxy[0] = rec.g_x;
   pxy[1] = rec.g_y;
   pxy[2] = rec.g_x + rec.g_w - 1;
   pxy[3] = rec.g_y + rec.g_h - 1;
   vr_recfl ( handle, pxy );
   graf_mouse ( M_ON, 0L );
}


/****************************************************************
* open_vwork
*
* This function opens a virtual workstation.
****************************************************************/
open_vwork ()
{
   int i;

   handle = graf_handle ( &charw, &charh, &dum, &dum);
   for ( i=0; i<10; work_in[i++] = 1 );
   work_in[10] = 2;
   v_opnvwk ( work_in, &handle, work_out );
}


/****************************************************************
* get_tedinfo_str
*
* This function calculates the address of an object's string.
****************************************************************/
char *get_tedinfo_str ( tree, object )
OBJECT *tree;
int object;
{
   TEDINFO *ob_tedinfo;

   ob_tedinfo = (TEDINFO *) tree[object].ob_spec;
   return ( ob_tedinfo->te_ptext );
}
